import requests
from jose.jwt import decode as jose_decode

def _get_jwks(jwks_url):
    resp = requests.get(jwks_url)
    return resp.json()


def decode(
    token,
    audience,
    jwks_url,
    **kwargs
):
    algorithms = ['RS256']
    key = _get_jwks(jwks_url)

    return jose_decode(
        token,
        key,
        algorithms=algorithms,
        audience=audience,
        **kwargs
    )



import requests
from social_core.backends.oauth import BaseOAuth2
from django_auth0_backend.jwt import decode
try:
    from nameparser import HumanName
except ImportError:
    pass

CLAIMS_PREFIX = 'https://{auth_domain}/claims/'
JWKS_URL = 'https://{auth0_domain}/.well-known/jwks.json'


class Auth0Backend(BaseOAuth2):
    name = 'auth0'
    SCOPE_SEPARATOR = ' '
    ACCESS_TOKEN_METHOD = 'POST'
    EXTRA_DATA = [
        ('picture', 'picture'),
        ('email_verified', 'email_verified'),
        ('claims', 'claims'),
    ]

    def authorization_url(self):
        return "https://{auth0_domain}/authorize".format(
            auth0_domain=self.setting('DOMAIN'),
        )

    def access_token_url(self):
        return "https://{auth0_domain}/oauth/token".format(
            auth0_domain=self.setting('DOMAIN'),
        )

    def get_user_id(self, details, response):
        return details['user_id']

    def _get_name(self, userinfo):
        if all(key in userinfo for key in ('given_name', 'family_name')):
            return userinfo['given_name'], userinfo['family_name']
        if 'name' in userinfo:
            try: 
                name = HumanName(userinfo['name'])
                return name.first, name.last
            except Exception:
                pass
        return userinfo['nickname'], ''


    def get_user_details(self, response):
        url = "https://{auth0_domain}/userinfo".format(
            auth0_domain=self.setting('DOMAIN'),
        )
        headers = {'Authorization': "Bearer {access_token}".format(
            access_token=response['access_token'],
        )}
        resp = requests.get(url, headers=headers)
        userinfo = resp.json()
        jwks_url = JWKS_URL.format(
            auth0_domain=self.setting('DOMAIN'),
        )
        userinfo = decode(
            response['id_token'],
            self.setting('KEY'),
            jwks_url,
        )
        claims = {
            k[len(CLAIMS_PREFIX):]: v
            for k, v in userinfo.items()
            if k.startswith(CLAIMS_PREFIX)
        }
        first_name, last_name = self._get_name(userinfo)
        return {
            'claims': claims,
            'username': userinfo['nickname'],
            'first_name': first_name,
            'last_name': last_name,
            'picture': userinfo['picture'],
            'user_id': userinfo['sub'],
            'email': userinfo['email'],
            'email_verified': userinfo.get('email_verified', None),

        }



from setuptools import setup

setup(
    name='django-auth0-backend',
    author = "Bryan Marty",
    author_email = "bryan@bryanmarty.com",
    keywords = "django python social auth auth0",
    version='0.2.0',
    packages=['django_auth0_backend'],
    license='MIT License',
    install_requires=[
        'python-jose>=3.0.0',
        'requests>=2.22.0',
        'social-auth-core>=3.2.0',
    ],
    extra_requires=[
        'nameparser',
        'cryptography',
    ],
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
    ],
)
